@@ -21,7 +21,6 @@
 
 #include "argv.h"
 #include "client.h"
-#include "common/recording.h"
 #include "io.h"
 #include "kubernetes.h"
 #include "ssl.h"
@@ -30,6 +29,7 @@
 
 #include <guacamole/client.h>
 #include <guacamole/protocol.h>
+#include <guacamole/recording.h>
 #include <libwebsockets.h>
 
 #include <pthread.h>
@@ -213,10 +213,11 @@ void* guac_kubernetes_client_thread(void* data) {
     }
 
     /* Generate endpoint for attachment URL */
-    if (guac_kubernetes_endpoint_attach(endpoint_path, sizeof(endpoint_path),
+    if (guac_kubernetes_endpoint_uri(endpoint_path, sizeof(endpoint_path),
                 settings->kubernetes_namespace,
                 settings->kubernetes_pod,
-                settings->kubernetes_container)) {
+                settings->kubernetes_container,
+                settings->exec_command)) {
         guac_client_abort(client, GUAC_PROTOCOL_STATUS_SERVER_ERROR,
                 "Unable to generate path for Kubernetes API endpoint: "
                 "Resulting path too long");
@@ -228,21 +229,33 @@ void* guac_kubernetes_client_thread(void* data) {
 
     /* Set up screen recording, if requested */
     if (settings->recording_path != NULL) {
-        kubernetes_client->recording = guac_common_recording_create(client,
+        kubernetes_client->recording = guac_recording_create(client,
                 settings->recording_path,
                 settings->recording_name,
                 settings->create_recording_path,
                 !settings->recording_exclude_output,
                 !settings->recording_exclude_mouse,
+                0, /* Touch events not supported */
                 settings->recording_include_keys);
     }
 
+    /* Create terminal options with required parameters */
+    guac_terminal_options* options = guac_terminal_options_create(
+            settings->width, settings->height, settings->resolution);
+
+    /* Set optional parameters */
+    options->disable_copy = settings->disable_copy;
+    options->max_scrollback = settings->max_scrollback;
+    options->font_name = settings->font_name;
+    options->font_size = settings->font_size;
+    options->color_scheme = settings->color_scheme;
+    options->backspace = settings->backspace;
+
     /* Create terminal */
-    kubernetes_client->term = guac_terminal_create(client,
-            kubernetes_client->clipboard, settings->disable_copy,
-            settings->max_scrollback, settings->font_name, settings->font_size,
-            settings->resolution, settings->width, settings->height,
-            settings->color_scheme, settings->backspace);
+    kubernetes_client->term = guac_terminal_create(client, options);
+
+    /* Free options struct now that it's been used */
+    free(options);
 
     /* Fail if terminal init failed */
     if (kubernetes_client->term == NULL) {
@@ -356,7 +369,7 @@ fail:
 
     /* Clean up recording, if in progress */
     if (kubernetes_client->recording != NULL)
-        guac_common_recording_free(kubernetes_client->recording);
+        guac_recording_free(kubernetes_client->recording);
 
     /* Free WebSocket context if successfully allocated */
     if (kubernetes_client->context != NULL)
@@ -400,8 +413,8 @@ void guac_kubernetes_force_redraw(guac_client* client) {
 
     /* Get current terminal dimensions */
     guac_terminal* term = kubernetes_client->term;
-    int rows = term->term_height;
-    int columns = term->term_width;
+    int rows = guac_terminal_get_rows(term);
+    int columns = guac_terminal_get_columns(term);
 
     /* Force a redraw by increasing the terminal size by one character in
      * each dimension and then resizing it back to normal (the same technique
