@@ -24,17 +24,6 @@
 
 #include <stdbool.h>
 
-/**
- * The name of the font to use for the terminal if no name is specified.
- */
-#define GUAC_KUBERNETES_DEFAULT_FONT_NAME "monospace" 
-
-/**
- * The size of the font to use for the terminal if no font size is specified,
- * in points.
- */
-#define GUAC_KUBERNETES_DEFAULT_FONT_SIZE 12
-
 /**
  * The port to connect to when initiating any Kubernetes connection, if no
  * other port is specified.
@@ -57,11 +46,6 @@
  */
 #define GUAC_KUBERNETES_DEFAULT_RECORDING_NAME "recording"
 
-/**
- * The default maximum scrollback size in rows.
- */
-#define GUAC_KUBERNETES_DEFAULT_MAX_SCROLLBACK 1000
-
 /**
  * Settings for the Kubernetes connection. The values for this structure are
  * parsed from the arguments given during the Guacamole protocol handshake
@@ -97,6 +81,12 @@ typedef struct guac_kubernetes_settings {
      */
     char* kubernetes_container;
 
+    /**
+     * The command to generate api endpoint for call exec. 
+     * If omitted call attach will be used.
+     */
+    char* exec_command;
+
     /**
      * Whether SSL/TLS should be used.
      */
