@@ -76,6 +76,9 @@ static UINT guac_rdp_cliprdr_send_format_list(CliprdrClientContext* cliprdr) {
     guac_rdp_clipboard* clipboard = (guac_rdp_clipboard*) cliprdr->custom;
     assert(clipboard != NULL);
 
+    guac_client* client = clipboard->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+
     /* We support CP-1252 and UTF-16 text */
     CLIPRDR_FORMAT_LIST format_list = {
         .msgType = CB_FORMAT_LIST,
@@ -86,10 +89,12 @@ static UINT guac_rdp_cliprdr_send_format_list(CliprdrClientContext* cliprdr) {
         .numFormats = 2
     };
 
-    guac_client_log(clipboard->client, GUAC_LOG_TRACE, "CLIPRDR: Sending "
-            "format list");
+    guac_client_log(client, GUAC_LOG_TRACE, "CLIPRDR: Sending format list");
 
-    return cliprdr->ClientFormatList(cliprdr, &format_list);
+    pthread_mutex_lock(&(rdp_client->message_lock));
+    int retval = cliprdr->ClientFormatList(cliprdr, &format_list);
+    pthread_mutex_unlock(&(rdp_client->message_lock));
+    return retval;
 
 }
 
@@ -107,6 +112,17 @@ static UINT guac_rdp_cliprdr_send_format_list(CliprdrClientContext* cliprdr) {
  */
 static UINT guac_rdp_cliprdr_send_capabilities(CliprdrClientContext* cliprdr) {
 
+    /* This function is only invoked within FreeRDP-specific handlers for
+     * CLIPRDR, which are not assigned, and thus not callable, until after the
+     * relevant guac_rdp_clipboard structure is allocated and associated with
+     * the CliprdrClientContext */
+    guac_rdp_clipboard* clipboard = (guac_rdp_clipboard*) cliprdr->custom;
+    assert(clipboard != NULL);
+
+    guac_client* client = clipboard->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+
+    /* We support CP-1252 and UTF-16 text */
     CLIPRDR_GENERAL_CAPABILITY_SET cap_set = {
         .capabilitySetType = CB_CAPSTYPE_GENERAL, /* CLIPRDR specification requires that this is CB_CAPSTYPE_GENERAL, the only defined set type */
         .capabilitySetLength = 12, /* The size of the capability set within the PDU - for CB_CAPSTYPE_GENERAL, this is ALWAYS 12 bytes */
@@ -119,7 +135,11 @@ static UINT guac_rdp_cliprdr_send_capabilities(CliprdrClientContext* cliprdr) {
         .capabilitySets = (CLIPRDR_CAPABILITY_SET*) &cap_set
     };
 
-    return cliprdr->ClientCapabilities(cliprdr, &caps);
+    pthread_mutex_lock(&(rdp_client->message_lock));
+    int retval = cliprdr->ClientCapabilities(cliprdr, &caps);
+    pthread_mutex_unlock(&(rdp_client->message_lock));
+
+    return retval;
 
 }
 
@@ -190,6 +210,9 @@ static UINT guac_rdp_cliprdr_send_format_data_request(
     guac_rdp_clipboard* clipboard = (guac_rdp_clipboard*) cliprdr->custom;
     assert(clipboard != NULL);
 
+    guac_client* client = clipboard->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+
     /* Create new data request */
     CLIPRDR_FORMAT_DATA_REQUEST data_request = {
         .requestedFormatId = format
@@ -199,11 +222,14 @@ static UINT guac_rdp_cliprdr_send_format_data_request(
      * data is received via a Format Data Response PDU */
     clipboard->requested_format = format;
 
-    guac_client_log(clipboard->client, GUAC_LOG_TRACE, "CLIPRDR: Sending "
-            "format data request.");
+    guac_client_log(client, GUAC_LOG_TRACE, "CLIPRDR: Sending format data request.");
 
     /* Send request */
-    return cliprdr->ClientFormatDataRequest(cliprdr, &data_request);
+    pthread_mutex_lock(&(rdp_client->message_lock));
+    int retval = cliprdr->ClientFormatDataRequest(cliprdr, &data_request);
+    pthread_mutex_unlock(&(rdp_client->message_lock));
+
+    return retval;
 
 }
 
@@ -265,15 +291,19 @@ static UINT guac_rdp_cliprdr_format_list(CliprdrClientContext* cliprdr,
     guac_rdp_clipboard* clipboard = (guac_rdp_clipboard*) cliprdr->custom;
     assert(clipboard != NULL);
 
-    guac_client_log(clipboard->client, GUAC_LOG_TRACE, "CLIPRDR: Received "
-            "format list.");
+    guac_client* client = clipboard->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+
+    guac_client_log(client, GUAC_LOG_TRACE, "CLIPRDR: Received format list.");
 
     CLIPRDR_FORMAT_LIST_RESPONSE format_list_response = {
         .msgFlags = CB_RESPONSE_OK
     };
 
     /* Report successful processing of format list */
+    pthread_mutex_lock(&(rdp_client->message_lock));
     cliprdr->ClientFormatListResponse(cliprdr, &format_list_response);
+    pthread_mutex_unlock(&(rdp_client->message_lock));
 
     /* Prefer Unicode (in this case, UTF-16) */
     if (guac_rdp_cliprdr_format_supported(format_list, CF_UNICODETEXT))
@@ -284,9 +314,9 @@ static UINT guac_rdp_cliprdr_format_list(CliprdrClientContext* cliprdr,
         return guac_rdp_cliprdr_send_format_data_request(cliprdr, CF_TEXT);
 
     /* Ignore any unsupported data */
-    guac_client_log(clipboard->client, GUAC_LOG_DEBUG, "Ignoring unsupported "
-            "clipboard data. Only Unicode and text clipboard formats are "
-            "currently supported.");
+    guac_client_log(client, GUAC_LOG_DEBUG, "Ignoring unsupported clipboard "
+            "data. Only Unicode and text clipboard formats are currently "
+            "supported.");
 
     return CHANNEL_RC_OK;
 
@@ -320,32 +350,35 @@ static UINT guac_rdp_cliprdr_format_data_request(CliprdrClientContext* cliprdr,
     guac_rdp_clipboard* clipboard = (guac_rdp_clipboard*) cliprdr->custom;
     assert(clipboard != NULL);
 
-    guac_client_log(clipboard->client, GUAC_LOG_TRACE, "CLIPRDR: Received "
-            "format data request.");
+    guac_client* client = clipboard->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+    guac_rdp_settings* settings = rdp_client->settings;
 
-    guac_iconv_write* writer;
+    guac_client_log(client, GUAC_LOG_TRACE, "CLIPRDR: Received format data request.");
+
+    guac_iconv_write* remote_writer;
     const char* input = clipboard->clipboard->buffer;
-    char* output = malloc(GUAC_RDP_CLIPBOARD_MAX_LENGTH);
+    char* output = malloc(GUAC_COMMON_CLIPBOARD_MAX_LENGTH);
 
     /* Map requested clipboard format to a guac_iconv writer */
     switch (format_data_request->requestedFormatId) {
 
         case CF_TEXT:
-            writer = GUAC_WRITE_CP1252;
+            remote_writer = settings->clipboard_crlf ? GUAC_WRITE_CP1252_CRLF : GUAC_WRITE_CP1252;
             break;
 
         case CF_UNICODETEXT:
-            writer = GUAC_WRITE_UTF16;
+            remote_writer = settings->clipboard_crlf ? GUAC_WRITE_UTF16_CRLF : GUAC_WRITE_UTF16;
             break;
 
         /* Warn if clipboard data cannot be sent as intended due to a violation
          * of the CLIPRDR spec */
         default:
-            guac_client_log(clipboard->client, GUAC_LOG_WARNING, "Received "
-                    "clipboard data cannot be sent to the RDP server because "
-                    "the RDP server has requested a clipboard format which "
-                    "was not declared as available. This violates the "
-                    "specification for the CLIPRDR channel.");
+            guac_client_log(client, GUAC_LOG_WARNING, "Received clipboard "
+                    "data cannot be sent to the RDP server because the RDP "
+                    "server has requested a clipboard format which was not "
+                    "declared as available. This violates the specification "
+                    "for the CLIPRDR channel.");
             free(output);
             return CHANNEL_RC_OK;
 
@@ -354,8 +387,9 @@ static UINT guac_rdp_cliprdr_format_data_request(CliprdrClientContext* cliprdr,
     /* Send received clipboard data to the RDP server in the format
      * requested */
     BYTE* start = (BYTE*) output;
-    guac_iconv(GUAC_READ_UTF8, &input, clipboard->clipboard->length,
-               writer, &output, GUAC_RDP_CLIPBOARD_MAX_LENGTH);
+    guac_iconv_read* local_reader = settings->normalize_clipboard ? GUAC_READ_UTF8_NORMALIZED : GUAC_READ_UTF8;
+    guac_iconv(local_reader, &input, clipboard->clipboard->length,
+            remote_writer, &output, GUAC_COMMON_CLIPBOARD_MAX_LENGTH);
 
     CLIPRDR_FORMAT_DATA_RESPONSE data_response = {
         .requestedFormatData = (BYTE*) start,
@@ -363,10 +397,12 @@ static UINT guac_rdp_cliprdr_format_data_request(CliprdrClientContext* cliprdr,
         .msgFlags = CB_RESPONSE_OK
     };
 
-    guac_client_log(clipboard->client, GUAC_LOG_TRACE, "CLIPRDR: Sending "
-            "format data response.");
+    guac_client_log(client, GUAC_LOG_TRACE, "CLIPRDR: Sending format data response.");
 
+    pthread_mutex_lock(&(rdp_client->message_lock));
     UINT result = cliprdr->ClientFormatDataResponse(cliprdr, &data_response);
+    pthread_mutex_unlock(&(rdp_client->message_lock));
+
     free(start);
     return result;
 
@@ -407,15 +443,15 @@ static UINT guac_rdp_cliprdr_format_data_response(CliprdrClientContext* cliprdr,
 
     /* Ignore received data if copy has been disabled */
     if (settings->disable_copy) {
-        guac_client_log(clipboard->client, GUAC_LOG_DEBUG, "Ignoring received "
-                "clipboard data as copying from within the remote desktop has "
-                "been explicitly disabled.");
+        guac_client_log(client, GUAC_LOG_DEBUG, "Ignoring received clipboard "
+                "data as copying from within the remote desktop has been "
+                "explicitly disabled.");
         return CHANNEL_RC_OK;
     }
 
-    char received_data[GUAC_RDP_CLIPBOARD_MAX_LENGTH];
+    char received_data[GUAC_COMMON_CLIPBOARD_MAX_LENGTH];
 
-    guac_iconv_read* reader;
+    guac_iconv_read* remote_reader;
     const char* input = (char*) format_data_response->requestedFormatData;
     char* output = received_data;
 
@@ -424,12 +460,12 @@ static UINT guac_rdp_cliprdr_format_data_response(CliprdrClientContext* cliprdr,
 
         /* Non-Unicode (Windows CP-1252) */
         case CF_TEXT:
-            reader = GUAC_READ_CP1252;
+            remote_reader = settings->normalize_clipboard ? GUAC_READ_CP1252_NORMALIZED : GUAC_READ_CP1252;
             break;
 
         /* Unicode (UTF-16) */
         case CF_UNICODETEXT:
-            reader = GUAC_READ_UTF16;
+            remote_reader = settings->normalize_clipboard ? GUAC_READ_UTF16_NORMALIZED : GUAC_READ_UTF16;
             break;
 
         /* If the format ID stored within the guac_rdp_clipboard structure is actually
@@ -439,21 +475,20 @@ static UINT guac_rdp_cliprdr_format_data_response(CliprdrClientContext* cliprdr,
          * here. The values which may be stored within requested_format are
          * completely within our control. */
         default:
-            guac_client_log(clipboard->client, GUAC_LOG_DEBUG, "Requested "
-                    "clipboard data in unsupported format (0x%X).",
-                    clipboard->requested_format);
+            guac_client_log(client, GUAC_LOG_DEBUG, "Requested clipboard data "
+                    "in unsupported format (0x%X).", clipboard->requested_format);
             return CHANNEL_RC_OK;
 
     }
 
     /* Convert, store, and forward the clipboard data received from RDP
      * server */
-    if (guac_iconv(reader, &input, format_data_response->dataLen,
+    if (guac_iconv(remote_reader, &input, format_data_response->dataLen,
             GUAC_WRITE_UTF8, &output, sizeof(received_data))) {
         int length = strnlen(received_data, sizeof(received_data));
         guac_common_clipboard_reset(clipboard->clipboard, "text/plain");
         guac_common_clipboard_append(clipboard->clipboard, received_data, length);
-        guac_common_clipboard_send(clipboard->clipboard, clipboard->client);
+        guac_common_clipboard_send(clipboard->clipboard, client);
     }
 
     return CHANNEL_RC_OK;
@@ -513,12 +548,54 @@ static void guac_rdp_cliprdr_channel_connected(rdpContext* context,
 
 }
 
+/**
+ * Callback which disassociates Guacamole from the CliprdrClientContext
+ * instance that was originally allocated by FreeRDP and is about to be
+ * deallocated.
+ *
+ * This function is called whenever a channel disconnects via the PubSub event
+ * system within FreeRDP, but only has any effect if the disconnected channel
+ * is the CLIPRDR channel. This specific callback is registered with the PubSub
+ * system of the relevant rdpContext when guac_rdp_clipboard_load_plugin() is
+ * called.
+ *
+ * @param context
+ *     The rdpContext associated with the active RDP session.
+ *
+ * @param e
+ *     Event-specific arguments, mainly the name of the channel, and a
+ *     reference to the associated plugin loaded for that channel by FreeRDP.
+ */
+static void guac_rdp_cliprdr_channel_disconnected(rdpContext* context,
+        ChannelDisconnectedEventArgs* e) {
+
+    guac_client* client = ((rdp_freerdp_context*) context)->client;
+    guac_rdp_client* rdp_client = (guac_rdp_client*) client->data;
+    guac_rdp_clipboard* clipboard = rdp_client->clipboard;
+
+    /* FreeRDP-specific handlers for CLIPRDR are not assigned, and thus not
+     * callable, until after the relevant guac_rdp_clipboard structure is
+     * allocated and associated with the guac_rdp_client */
+    assert(clipboard != NULL);
+
+    /* Ignore disconnection event if it's not for the CLIPRDR channel */
+    if (strcmp(e->name, CLIPRDR_SVC_CHANNEL_NAME) != 0)
+        return;
+
+    /* Channel is no longer connected */
+    clipboard->cliprdr = NULL;
+
+    guac_client_log(client, GUAC_LOG_DEBUG, "CLIPRDR (clipboard redirection) "
+            "channel disconnected.");
+
+}
+
 guac_rdp_clipboard* guac_rdp_clipboard_alloc(guac_client* client) {
 
     /* Allocate clipboard and underlying storage */
     guac_rdp_clipboard* clipboard = calloc(1, sizeof(guac_rdp_clipboard));
     clipboard->client = client;
-    clipboard->clipboard = guac_common_clipboard_alloc(GUAC_RDP_CLIPBOARD_MAX_LENGTH);
+    clipboard->clipboard = guac_common_clipboard_alloc();
     clipboard->requested_format = CF_TEXT;
 
     return clipboard;
@@ -542,6 +619,10 @@ void guac_rdp_clipboard_load_plugin(guac_rdp_clipboard* clipboard,
     PubSub_SubscribeChannelConnected(context->pubSub,
             (pChannelConnectedEventHandler) guac_rdp_cliprdr_channel_connected);
 
+    /* Clean up any RDP-specific resources when channel is disconnected */
+    PubSub_SubscribeChannelDisconnected(context->pubSub,
+            (pChannelDisconnectedEventHandler) guac_rdp_cliprdr_channel_disconnected);
+
     guac_client_log(clipboard->client, GUAC_LOG_DEBUG, "Support for CLIPRDR "
             "(clipboard redirection) registered. Awaiting channel "
             "connection.");
