@@ -21,7 +21,6 @@
 #include "channels/audio-input/audio-buffer.h"
 #include "channels/cliprdr.h"
 #include "channels/disp.h"
-#include "common/recording.h"
 #include "config.h"
 #include "fs.h"
 #include "log.h"
@@ -37,6 +36,7 @@
 
 #include <guacamole/audio.h>
 #include <guacamole/client.h>
+#include <guacamole/recording.h>
 
 #include <dirent.h>
 #include <errno.h>
@@ -145,7 +145,10 @@ int guac_client_init(guac_client* client, int argc, char** argv) {
     rdp_client->clipboard = guac_rdp_clipboard_alloc(client);
 
     /* Init display update module */
-    rdp_client->disp = guac_rdp_disp_alloc();
+    rdp_client->disp = guac_rdp_disp_alloc(client);
+
+    /* Init multi-touch support module (RDPEI) */
+    rdp_client->rdpei = guac_rdp_rdpei_alloc(client);
 
     /* Redirect FreeRDP log messages to guac_client_log() */
     guac_rdp_redirect_wlog(client);
@@ -155,12 +158,14 @@ int guac_client_init(guac_client* client, int argc, char** argv) {
     pthread_mutexattr_settype(&(rdp_client->attributes),
             PTHREAD_MUTEX_RECURSIVE);
 
-    /* Initalize the lock */
+    /* Init required locks */
     pthread_rwlock_init(&(rdp_client->lock), NULL);
+    pthread_mutex_init(&(rdp_client->message_lock), &(rdp_client->attributes));
 
     /* Set handlers */
     client->join_handler = guac_rdp_user_join_handler;
     client->free_handler = guac_rdp_client_free_handler;
+    client->leave_handler = guac_rdp_user_leave_handler;
 
 #ifdef ENABLE_COMMON_SSH
     guac_common_ssh_init(client);
@@ -187,10 +192,21 @@ int guac_rdp_client_free_handler(guac_client* client) {
     /* Free display update module */
     guac_rdp_disp_free(rdp_client->disp);
 
+    /* Free multi-touch support module (RDPEI) */
+    guac_rdp_rdpei_free(rdp_client->rdpei);
+
     /* Clean up filesystem, if allocated */
     if (rdp_client->filesystem != NULL)
         guac_rdp_fs_free(rdp_client->filesystem);
 
+    /* End active print job, if any */
+    guac_rdp_print_job* job = (guac_rdp_print_job*) rdp_client->active_job;
+    if (job != NULL) {
+        guac_rdp_print_job_kill(job);
+        guac_rdp_print_job_free(job);
+        rdp_client->active_job = NULL;
+    }
+
 #ifdef ENABLE_COMMON_SSH
     /* Free SFTP filesystem, if loaded */
     if (rdp_client->sftp_filesystem)
@@ -209,7 +225,7 @@ int guac_rdp_client_free_handler(guac_client* client) {
 
     /* Clean up recording, if in progress */
     if (rdp_client->recording != NULL)
-        guac_common_recording_free(rdp_client->recording);
+        guac_recording_free(rdp_client->recording);
 
     /* Clean up audio stream, if allocated */
     if (rdp_client->audio != NULL)
@@ -220,6 +236,7 @@ int guac_rdp_client_free_handler(guac_client* client) {
         guac_rdp_audio_buffer_free(rdp_client->audio_input);
 
     pthread_rwlock_destroy(&(rdp_client->lock));
+    pthread_mutex_destroy(&(rdp_client->message_lock));
 
     /* Free client data */
     free(rdp_client);
