@@ -104,8 +104,9 @@ static void guac_rdp_send_key_event(guac_rdp_client* rdp_client,
         return;
 
     /* Send actual key */
-    rdp_inst->input->KeyboardEvent(rdp_inst->input,
-            flags | pressed_flags, scancode);
+    pthread_mutex_lock(&(rdp_client->message_lock));
+    rdp_inst->input->KeyboardEvent(rdp_inst->input, flags | pressed_flags, scancode);
+    pthread_mutex_unlock(&(rdp_client->message_lock));
 
 }
 
@@ -132,14 +133,14 @@ static void guac_rdp_send_unicode_event(guac_rdp_client* rdp_client,
         return;
 
     /* Send Unicode event */
-    rdp_inst->input->UnicodeKeyboardEvent(
-            rdp_inst->input,
-            0, codepoint);
+    pthread_mutex_lock(&(rdp_client->message_lock));
+    rdp_inst->input->UnicodeKeyboardEvent(rdp_inst->input, 0, codepoint);
+    pthread_mutex_unlock(&(rdp_client->message_lock));
 
 }
 
 /**
- * Immediately sends an RDP synchonize event having the given flags. An RDP
+ * Immediately sends an RDP synchronize event having the given flags. An RDP
  * synchronize event sets the state of remote lock keys absolutely, where a
  * lock key will be active only if its corresponding flag is set in the event.
  *
@@ -161,7 +162,9 @@ static void guac_rdp_send_synchronize_event(guac_rdp_client* rdp_client,
         return;
 
     /* Synchronize lock key states */
+    pthread_mutex_lock(&(rdp_client->message_lock));
     rdp_inst->input->SynchronizeEvent(rdp_inst->input, flags);
+    pthread_mutex_unlock(&(rdp_client->message_lock));
 
 }
 
