@@ -23,6 +23,7 @@
 #include "client.h"
 #include "common/defaults.h"
 #include "settings.h"
+#include "terminal/terminal.h"
 
 #include <guacamole/user.h>
 #include <guacamole/wol-constants.h>
@@ -72,6 +73,7 @@ const char* GUAC_SSH_CLIENT_ARGS[] = {
     "wol-send-packet",
     "wol-mac-addr",
     "wol-broadcast-addr",
+    "wol-udp-port",
     "wol-wait-time",
     NULL
 };
@@ -247,8 +249,8 @@ enum SSH_ARGS_IDX {
 
     /**
      * The ASCII code, as an integer, to send for the backspace key, as configured
-     * by the SSH connection from the client.  By default this will be 127,
-     * the ASCII DELETE code.
+     * by the SSH connection from the client.  By default this will be
+     * GUAC_TERMINAL_DEFAULT_BACKSPACE.
      */
     IDX_BACKSPACE,
 
@@ -317,6 +319,11 @@ enum SSH_ARGS_IDX {
      */
     IDX_WOL_BROADCAST_ADDR,
     
+    /**
+     * The UDP port to use when sending the WoL packet. 
+     */
+    IDX_WOL_UDP_PORT,
+    
     /**
      * The amount of time to wait after sending the magic WoL packet prior to
      * continuing the connection attempt.  The default is no wait time
@@ -369,22 +376,22 @@ guac_ssh_settings* guac_ssh_parse_args(guac_user* user,
     /* Read maximum scrollback size */
     settings->max_scrollback =
         guac_user_parse_args_int(user, GUAC_SSH_CLIENT_ARGS, argv,
-                IDX_SCROLLBACK, GUAC_SSH_DEFAULT_MAX_SCROLLBACK);
+                IDX_SCROLLBACK, GUAC_TERMINAL_DEFAULT_MAX_SCROLLBACK);
 
     /* Read font name */
     settings->font_name =
         guac_user_parse_args_string(user, GUAC_SSH_CLIENT_ARGS, argv,
-                IDX_FONT_NAME, GUAC_SSH_DEFAULT_FONT_NAME);
+                IDX_FONT_NAME, GUAC_TERMINAL_DEFAULT_FONT_NAME);
 
     /* Read font size */
     settings->font_size =
         guac_user_parse_args_int(user, GUAC_SSH_CLIENT_ARGS, argv,
-                IDX_FONT_SIZE, GUAC_SSH_DEFAULT_FONT_SIZE);
+                IDX_FONT_SIZE, GUAC_TERMINAL_DEFAULT_FONT_SIZE);
 
     /* Copy requested color scheme */
     settings->color_scheme =
         guac_user_parse_args_string(user, GUAC_SSH_CLIENT_ARGS, argv,
-                IDX_COLOR_SCHEME, "");
+                IDX_COLOR_SCHEME, GUAC_TERMINAL_DEFAULT_COLOR_SCHEME);
 
     /* Pull width/height/resolution directly from user */
     settings->width      = user->info.optimal_width;
@@ -485,7 +492,7 @@ guac_ssh_settings* guac_ssh_parse_args(guac_user* user,
     /* Parse backspace key setting */
     settings->backspace =
         guac_user_parse_args_int(user, GUAC_SSH_CLIENT_ARGS, argv,
-                IDX_BACKSPACE, 127);
+                IDX_BACKSPACE, GUAC_TERMINAL_DEFAULT_BACKSPACE);
 
     /* Read terminal emulator type. */
     settings->terminal_type =
@@ -533,6 +540,10 @@ guac_ssh_settings* guac_ssh_parse_args(guac_user* user,
             guac_user_parse_args_string(user, GUAC_SSH_CLIENT_ARGS, argv,
                 IDX_WOL_BROADCAST_ADDR, GUAC_WOL_LOCAL_IPV4_BROADCAST);
         
+        settings->wol_udp_port = (unsigned short)
+            guac_user_parse_args_int(user, GUAC_SSH_CLIENT_ARGS, argv,
+                IDX_WOL_UDP_PORT, GUAC_WOL_PORT);
+        
         settings->wol_wait_time =
             guac_user_parse_args_int(user, GUAC_SSH_CLIENT_ARGS, argv,
                 IDX_WOL_WAIT_TIME, GUAC_WOL_DEFAULT_BOOT_WAIT_TIME);
