@@ -26,6 +26,7 @@
 #include <guacamole/socket.h>
 #include <guacamole/user.h>
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -54,8 +55,9 @@ int guac_telnet_argv_callback(guac_user* user, const char* mimetype,
 
     /* Update terminal window size if connected */
     if (telnet_client->telnet != NULL && telnet_client->naws_enabled)
-        guac_telnet_send_naws(telnet_client->telnet, terminal->term_width,
-                terminal->term_height);
+        guac_telnet_send_naws(telnet_client->telnet,
+                guac_terminal_get_columns(terminal),
+                guac_terminal_get_rows(terminal));
 
     return 0;
 
@@ -68,15 +70,17 @@ void* guac_telnet_send_current_argv(guac_user* user, void* data) {
 
     /* Send current color scheme */
     guac_user_stream_argv(user, user->socket, "text/plain",
-            GUAC_TELNET_ARGV_COLOR_SCHEME, terminal->color_scheme);
+            GUAC_TELNET_ARGV_COLOR_SCHEME,
+            guac_terminal_get_color_scheme(terminal));
 
     /* Send current font name */
     guac_user_stream_argv(user, user->socket, "text/plain",
-            GUAC_TELNET_ARGV_FONT_NAME, terminal->font_name);
+            GUAC_TELNET_ARGV_FONT_NAME,
+            guac_terminal_get_font_name(terminal));
 
     /* Send current font size */
     char font_size[64];
-    sprintf(font_size, "%i", terminal->font_size);
+    sprintf(font_size, "%i", guac_terminal_get_font_size(terminal));
     guac_user_stream_argv(user, user->socket, "text/plain",
             GUAC_TELNET_ARGV_FONT_SIZE, font_size);
 
