@@ -19,7 +19,6 @@
 
 #include "config.h"
 
-#include "common/recording.h"
 #include "client.h"
 #include "user.h"
 #include "vnc.h"
@@ -35,6 +34,7 @@
 #endif
 
 #include <guacamole/client.h>
+#include <guacamole/recording.h>
 
 #include <pthread.h>
 #include <stdlib.h>
@@ -55,7 +55,7 @@ int guac_client_init(guac_client* client) {
 #endif
 
     /* Init clipboard */
-    vnc_client->clipboard = guac_common_clipboard_alloc(GUAC_VNC_CLIPBOARD_MAX_LENGTH);
+    vnc_client->clipboard = guac_common_clipboard_alloc();
 
     /* Set handlers */
     client->join_handler = guac_vnc_user_join_handler;
@@ -77,12 +77,27 @@ int guac_vnc_client_free_handler(guac_client* client) {
         /* Wait for client thread to finish */
         pthread_join(vnc_client->client_thread, NULL);
 
-        /* Free memory not free'd by libvncclient's rfbClientCleanup() */
-        if (rfb_client->frameBuffer != NULL) free(rfb_client->frameBuffer);
-        if (rfb_client->raw_buffer != NULL) free(rfb_client->raw_buffer);
-        if (rfb_client->rcSource != NULL) free(rfb_client->rcSource);
+        /* Free memory that may not be free'd by libvncclient's
+         * rfbClientCleanup() prior to libvncclient 0.9.12 */
+
+        if (rfb_client->frameBuffer != NULL) {
+            free(rfb_client->frameBuffer);
+            rfb_client->frameBuffer = NULL;
+        }
+
+        if (rfb_client->raw_buffer != NULL) {
+            free(rfb_client->raw_buffer);
+            rfb_client->raw_buffer = NULL;
+        }
+
+        if (rfb_client->rcSource != NULL) {
+            free(rfb_client->rcSource);
+            rfb_client->rcSource = NULL;
+        }
+
+        /* Free VNC rfbClientData linked list (may not be free'd by
+         * rfbClientCleanup(), depending on libvncclient version) */
 
-        /* Free VNC rfbClientData linked list (not free'd by rfbClientCleanup()) */
         while (rfb_client->clientData != NULL) {
             rfbClientData* next = rfb_client->clientData->next;
             free(rfb_client->clientData);
@@ -111,7 +126,7 @@ int guac_vnc_client_free_handler(guac_client* client) {
 
     /* Clean up recording, if in progress */
     if (vnc_client->recording != NULL)
-        guac_common_recording_free(vnc_client->recording);
+        guac_recording_free(vnc_client->recording);
 
     /* Free clipboard */
     if (vnc_client->clipboard != NULL)
