@@ -21,7 +21,12 @@
 #ifndef _GUAC_TERMINAL_DISPLAY_H
 #define _GUAC_TERMINAL_DISPLAY_H
 
-#include "config.h"
+/**
+ * Structures and function definitions related to the graphical display.
+ *
+ * @file display.h
+ */
+
 
 #include "common/surface.h"
 #include "palette.h"
